/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AddressbooksSynchronizer.
 *
 * The Initial Developer of the Original Code is Gnter Gersdorf.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Gnter Gersdorf <G.Gersdorf@ggbs.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
const DEBUG                           = false; // set to false to suppress debug messages

const MY_CONTRACTID     = "@ggbs.de/addressbooksSync;1";
const MY_NAME           = "Addressbooks Synchronizer";
const MY_CID            = Components.ID("{82057672-19A9-47E3-BB7f-0F08A28CE014}");

//loadSubScript (in wstring url[, targetObj]);

function nsAddressbooksSync()
{
  /* global variables */
  this.mCount         = null;
  this.mAppStarttime  = null; //in seconds (the javascript Date object is in milliseconds)
  this.mState         = null;
  this.mExiting       = false;
  this.mForceUpload   = null; //force upload after upgrading addressbookssync or enabling automatic upload
  this.mForceDownload = null; //force download after upgrading addressbookssync
  this.mForceSingleUL = new Object(); //force upload of single books, after selecting them for sync
  this.mStatusText    = null;
  this.mPendingUpload = false;

  /* preferences */
  this.mCheckLastModTime  = null;
  this.mSyncType          = null;
  this.mAutoUpload        = null;
  this.mAutoDownload      = null;
  this.mHidePopups        = null;
  this.mHideAllPopups     = null;
  this.mNoUpload          = null;
  this.mSeparateUpDown    = null;
  this.mTimedDownload     = null;
  this.mTimedUpload       = null;
  this.mLoadTimer         = null;
  this.mDownloadPolicy    = null;

  var prefService     = Components.classes["@mozilla.org/preferences-service;1"].
                          getService(Components.interfaces.nsIPrefService);
  this.mPrefBranch    = prefService.getBranch("extensions.addressbookssync.");

  if (this.mPrefBranch)
  {
    this.mPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this.mPrefBranch.addObserver("", this, false);
  }

  this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);

  this.loadTimer = Components.classes["@mozilla.org/timer;1"].
        createInstance(Components.interfaces.nsITimer);

  var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                      .getService(Components.interfaces.mozIJSSubScriptLoader);
  loader.loadSubScript ('chrome://addressbookssync/content/Utils.js', nsAddressbooksSync.prototype);
}

nsAddressbooksSync.prototype =
{
/************* global variables **************************/
  get count () {
    if (this.mCount == null)  this.mCount  = 0;
    return this.mCount;
  },
  set count (aCount) {
    this.mCount = aCount;
  }
,
  get appstarttime () {
    if (this.mAppStarttime == null)  this.mAppStarttime  = Math.round(Date.now()/1000);
    return this.mAppStarttime;
  },
  set appstarttime (aAppStarttime) {
    if (this.mAppStarttime==null) this.mAppStarttime = Math.round(Date.now()/1000);
    else if (aAppStarttime>0) this.mAppStarttime = aAppStarttime;
    return this.mAppStarttime;
  }
,
  get state () {
    if (this.mState == null)  this.mState  = '';
    return this.mState;
  },
  set state (aState) {
    if (this.mState == null)  this.mState  = '';
    if (aState=='')
      this.mState='';
    else if (this.mState=='')  // else Up-/Download in Progress!
      this.mState = aState;
  }
,
  get exiting () {
    return this.mExiting;
  },
  set exiting (aExiting) {
    this.mExiting = aExiting;
  }
,
  get forceUpload () {
    if (this.mForceUpload == null) {
      this.mForceUpload = false;
      x=this.checkLastModTime;
    }
    return this.mForceUpload;
  },
  set forceUpload (aForceUpload) {
    this.mForceUpload = aForceUpload;
  }
,
  get forceDownload () {
    if (this.mForceDownload == null) {
      this.mForceDownload = false;
      x=this.checkLastModTime;
    }
    return this.mForceDownload;
  },
  set forceDownload (aForceDownload) {
    this.mForceDownload = aForceDownload;
  }
,
  forceUploadOf: function(aBook, aSet)
  {
    this.mForceSingleUL[aBook]=aSet;
//    return this.mForceSingleUL.hasOwnProperty(aBook) && this.mForceSingleUL[aBook];
    return this.mForceSingleUL[aBook];
  }
,
  forcedUploadOf: function(aBook)
  {
    return this.mForceSingleUL.hasOwnProperty(aBook) && this.mForceSingleUL[aBook];
  }
,
  get statusText () {
    if (this.mStatusText == null)  this.mStatusText  = '';
    return this.mStatusText;
  },
  set statusText (aStatusText) {
    if (this.mStatusText == null)  this.mStatusText  = '';
    if (aStatusText==null)
      this.mStatusText='';
    else
      this.mStatusText += aStatusText;
  }
,
  get pendingupload () {
    var val=this.mPendingUpload;
    this.mPendingUpload=false;
    return val;
  },
  set pendingupload (aPendingUpload) {
    this.mPendingUpload = aPendingUpload;
  }
,
/************* preferences **************************/
  get checkLastModTime ()
  {
    if (this.mCheckLastModTime == null)
      try {
        this.mCheckLastModTime  = this.mPrefBranch.getBoolPref("checkLastModTime");
      } catch (e) {
        // if this preferences is not set, we are upgrading AddressbooksSync
        // => do a first upload of all books with timestamp!!
        this.checkLastModTime  = true;
        this.forceUpload = true;
        this.forceDownload = true;
        // set the 'upload always' flag
      }
    return this.mCheckLastModTime;
  }
,
  set checkLastModTime (aCheckLastModTime)
  {
    this.mPrefBranch.setBoolPref("checkLastModTime", aCheckLastModTime);
    this.mCheckLastModTime = aCheckLastModTime;
    return;
  }
,
  get synctype () {
    if (this.mSyncType==null) {
      try { this.mSyncType = this.mPrefBranch.getCharPref("synctype"); }
        catch(e) { this.mSyncType = 'none'; }
      if (this.mSyncType!='none' && this.mSyncType!='local'&&
          this.mSyncType!='remote' && this.mSyncType!='imap')
        this.mSyncType='none';
    }
    return this.mSyncType;
  },
  set synctype (aSyncType) {
    this.mPrefBranch.setCharPref("synctype", aSyncType);
    this.mSyncType = aSyncType;
  }
,
  get autoupload () {
    if (this.noupload) return false;
    if (this.mAutoUpload==null) {
      try { this.mAutoUpload = this.mPrefBranch.getBoolPref("autoupload"); }
        catch(e) { this.mAutoUpload = false; }
    }
    return this.mAutoUpload;
  },
  set autoupload (aAutoUpload) {
    if (this.noupload) return;
    this.mPrefBranch.setBoolPref("autoupload", aAutoUpload);
    this.mAutoUpload = aAutoUpload;
  }
,
  get autodownload () {
    if (this.mAutoDownload==null) {
      try { this.mAutoDownload = this.mPrefBranch.getBoolPref("autodownload"); }
        catch(e) { this.mAutoDownload = false; }
    }
    return this.mAutoDownload;
  },
  set autodownload (aAutoDownload) {
    this.mPrefBranch.setBoolPref("autodownload", aAutoDownload);
    this.mAutoDownload = aAutoDownload;
  }
,
  get hidepopups () {
    if (this.mHidePopups==null) {
      try { this.mHidePopups = this.mPrefBranch.getBoolPref("hidepopups"); }
        catch(e) { this.mHidePopups = false; }
    }
    return this.mHidePopups;
  },
  set hidepopups (aHidePopups) {
    this.mPrefBranch.setBoolPref("hidepopups", aHidePopups);
    this.mHidePopups = aHidePopups;
  }
,
 get hideallpopups () {
    if (this.mHideAllPopups==null) {
      try { this.mHideAllPopups = this.mPrefBranch.getBoolPref("hideallpopups"); }
        catch(e) { this.mHideAllPopups = false; }
    }
    return this.mHideAllPopups;
  },
  set hideallpopups (aHideAllPopups) {
    this.mPrefBranch.setBoolPref("hideallpopups", aHideAllPopups);
    this.mHideAllPopups = aHideAllPopups;
  }
,
  get noupload () {
    if (this.mNoUpload==null) {
      try { this.mNoUpload = this.mPrefBranch.getBoolPref("noupload"); }
        catch(e) { this.mNoUpload = false; }
    }
    return this.mNoUpload;
  },
  set noupload (aNoUpload) {
    this.mPrefBranch.setBoolPref("noupload", aNoUpload);
    this.mNoUpload = aNoUpload;
  }
,
  get separateupdown () {
    if (this.noupload) return false;
    if (this.mSeparateUpDown==null) {
      try { this.mSeparateUpDown = this.mPrefBranch.getBoolPref("separateupdown"); }
        catch(e) { this.mSeparateUpDown = false; }
    }
    return this.mSeparateUpDown;
  },
  set separateupdown (aSeparateUpDown) {
    if (this.noupload) return;
    this.mPrefBranch.setBoolPref("separateupdown", aSeparateUpDown);
    this.mSeparateUpDown = aSeparateUpDown;
  }
,
  get timeddownload () {
    if (this.mTimedDownload==null) {
      try { this.mTimedDownload = this.mPrefBranch.getBoolPref("timeddownload"); }
        catch(e) { this.mTimedDownload = false; }
    }
    return this.mTimedDownload;
  },
  set timeddownload (aTimedDownload) {
    this.mPrefBranch.setBoolPref("timeddownload", aTimedDownload);
    this.mTimedDownload = aTimedDownload;
  }
,
  get timedupload () {
    if (this.mTimedUpload==null) {
      try { this.mTimedUpload = this.mPrefBranch.getBoolPref("timedupload"); }
        catch(e) { this.mTimedUpload = false; }
    }
    return this.mTimedUpload;
  },
  set timedupload (aTimedUpload) {
    this.mPrefBranch.setIntPref("timedupload", aTimedUpload);
    this.mTimedUpload = aTimedUpload;
  }
,
  get loadtimer () {
    if (this.mLoadTimer==null) {
      try { this.mLoadTimer = this.mPrefBranch.getIntPref("loadtimer"); }
        catch(e) { this.mLoadTimer = 0; }
    }
    return this.mLoadTimer;
  },
  set loadtimer (aLoadTimer) {
    if (this.mLoadTimer != aLoadTimer)
      this.mPrefBranch.setIntPref("loadtimer", aLoadTimer);
    this.mLoadTimer = aLoadTimer;
    this.loadTimer.cancel();
    if (aLoadTimer && (this.timeddownload||this.timedupload))
      this.loadTimer.initWithCallback(this, aLoadTimer*60000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
  }
,
  get downloadpolicy () {
    if (this.mDownloadPolicy==null) {
      try { this.mDownloadPolicy = this.mPrefBranch.getCharPref("downloadpolicy"); }
        catch(e) { this.mDownloadPolicy = 'ask'; }
      if (this.mDownloadPolicy!='ask' && this.mDownloadPolicy!='overwrite' &&
          this.mDownloadPolicy!='keep')
        this.mDownloadPolicy!='ask'
    }
    return this.mDownloadPolicy;
  },
  set downloadpolicy (aDownloadPolicy) {
    this.mPrefBranch.setCharPref("downloadpolicy", aDownloadPolicy);
    this.mDownloadPolicy = aDownloadPolicy;
  }
,
/************ my functions *******************************/
  notify: function(aTimer)
  {
    var windowsEnum = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService()
                     .QueryInterface(Components.interfaces.nsIWindowMediator)
                     .getEnumerator(null);  //enumerate all windows //"mail:3pane");
    while (windowsEnum.hasMoreElements()) { //no more windows, this is the first! Sync!
      var w=windowsEnum.getNext();  // there must be one (the current)!
      if (w.showPopup) break;
      w=null;
    }
    if (!w) //no more windows, this was the last one! Sync!
      this.promptService.alert(null, 'info', 'no window to perform up-/download!');
    else {
      if (this.mTimedDownload) {
        if (this.mTimedUpload) this.mPendingUpload=true;
        w.showPopup('download', false, false, true, false);
      } else if (this.mTimedUpload)
        w.showPopup('upload', false, false, true, false);
    }
  },


/************* system functions **************************/
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case "nsPref:changed":
//dump('pref changed '+aSubject+' '+aData+'\n');
//TODO: eventuell preferences nicht als einzelwerte sondern als
//      attribute eines Objects speichern
        if (aData=='checkLastModTime') this.mCheckLastModTime  = null;
        if (aData=='synctype')         this.mSyncType          = null;
        if (aData=='autoupload')       this.mAutoUpload        = null;
        if (aData=='autodownload')     this.mAutoDownload      = null;
        if (aData=='hidepopups')       this.mHidePopups     = null;
        if (aData=='hideallpopups')    this.mHideAllPopups     = null;
        if (aData=='noupload')         this.mNoUpload          = null;
        break;
    }
  }
,

  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.AddressbooksSync) &&
//        !aIID.equals(Components.interfaces.nsIObserver) &&
        !aIID.equals(Components.interfaces.nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;

    return this;
  }
}

var addressbooksSyncModule =
{
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(MY_CID, MY_NAME, MY_CONTRACTID, aFileSpec, aLocation, aType);
  }
,
  unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(MY_CID, aFileSpec);
  }
,
  getClassObject: function (aCompMgr, aCID, aIID)
  {
    if (!aCID.equals(MY_CID))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.mFactory;
  }
,
  canUnload: function(aCompMgr)
  {
    return true;
  }
,
  /* factory object */
  mFactory:
  {
    createInstance: function(aOuter, aIID)
    {
      if (aOuter != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

      return (new nsAddressbooksSync()).QueryInterface(aIID);
    }
  ,
    lockFactory: function(aLock)
    {
      // quiten warnings
    }
  },
}

/* entrypoint */
function NSGetModule(aCompMgr, aFileSpec)
{
  return addressbooksSyncModule;
}

/* static functions */
if (DEBUG)
    debug = function (s) { dump("-*- AddressbooksSync component: " + s + "\n"); }
else
    debug = function (s) {}